#!/usr/bin/perl

# MERLIN_LOD2MAP.PL
#
# Merges LOD scores from "merlin.out" with marker information from map.<chr>.
# outfile: map_LOD.<chr>

# 2006 (c) Franz Ruschendorf


################################  V A R I A B L E S  ###########################

use strict;

my @v=();
my @output=();
my ($infile,$mapfile,$outfile,$chr);
my $score="LOD";
my %LOD=();
my %HLOD=();
my %npLOD=();

if ($ARGV[0] eq "-hlod") { $score = "HLOD"; }
	

#####################################  M A I N  ################################

my @chromes = qw(01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24);
foreach (@chromes) {
	$chr = $_;
	my $dir = "c${chr}";
	my $err=0;

	if ( -d $dir) {
		chdir "$dir";
		print "Read $dir: ";

		$infile="merlin.out";
		$mapfile="map.${chr}";	# markermap of set in chromosomefolder
		$outfile="map_LOD.$chr";	# Combined resultfile

		if (-f $infile) {&ReadMerlinLOD;} else {print " no infile $infile"; $err=1;}
		if (-f $mapfile) {&ReadMapFile; } else {print ", no mapfile $mapfile\n";$err=1;}

		if ($err ==0) {&PrintOutfile;}
		chdir "..";
	}
}



###########################  Read Merlin LOD scores  ###########################
sub ReadMerlinLOD {

	%LOD=();
	%HLOD=();
	my $c= 0;
	my $j=0;
	
	open (IN, "$infile") || print "could not open infile $infile :$!\n";
	print " $infile";

	while (<IN>){
		chomp();
		my @v = split(' ',$_);
		if ($v[0] eq "") {$c=-1;}
		if ($v[0] =~ /POSITION/) { $c=1; next;}
		if ($c > 0) {
			$c++;
			$LOD{$c}=$v[1];
			$HLOD{$c}=$v[3];
		}

		if ($v[0] eq "") {$j=-1;}
		if ($v[0] =~ /max/) { $j=1; next;}
		if ($j > 0) {
			$j++;
			$npLOD{$j}=$v[4];
		}





	}
	close (IN);
}

########################  Read Mapfile - Map of Set  ########################
sub ReadMapFile {
	my $string="";
	@output=();
	my $npL="";
	
	open (IN, "$mapfile") || die "could not open mapfile of set $mapfile :$!\n";
	print "  $mapfile";

	while (<IN>){

		chomp();
		if ($. == 1) { 
			$string="CHR\tSNP_ID\tPOSITION\tGenpos\tNr\tLOD\tHLOD\tnpLOD";
			push (@output,$string);
		}
		else {
			my @v = split(' ',$_);
			if (exists $npLOD{$.}) {$npL=$npLOD{$.};}
			else { $npL="---"; }

			$string="$v[0]\t$v[2]\t$v[3]\t$v[1]\t$v[4]\t$LOD{$.}\t$HLOD{$.}\t$npL";
			push (@output,$string);
		}
	}
	close (IN);
}

###################################  SUBs  ####################################
sub PrintOutfile {
	open (OUT, "> $outfile") || die "could not open outfile $outfile :$!\n";
	foreach (@output) {
		print OUT "$_\n";
	}
	close (OUT);
	print "\tWrite: $outfile\n";
	
}

######################################  E N D  #################################